# KInterbasDB Python Package - Python Wrapper for Core
#
# Version 3.1
#
# The following contributors hold Copyright (C) over their respective
# portions of code (see license.txt for details):
#
# [Original Author (maintained through version 2.0-0.3.1):]
#   1998-2001 [alex]  Alexander Kuznetsov   <alexan@users.sourceforge.net>
# [Maintainers (after version 2.0-0.3.1):]
#   2001-2002 [maz]   Marek Isalski         <kinterbasdb@maz.nu>
#   2002-2003 [dsr]   David Rushby          <woodsplitter@rocketmail.com>
# [Contributors:]
#   2001      [eac]   Evgeny A. Cherkashin  <eugeneai@icc.ru>
#   2001-2002 [janez] Janez Jere            <janez.jere@void.si>

__all__ = (
    # kinterbasdb-native fixed point converters (old, precison_mode style):
    'fixed_conv_in_imprecise', 'fixed_conv_in_precise',
    'fixed_conv_out_imprecise', 'fixed_conv_out_precise',
  )

import sys, types
from kinterbasdb.k_exceptions import *

if sys.version_info < (2,2):
    from kinterbasdb.typeconv_util_isinstance import isinstance

# The fixedpoint module resides at:  http://fixedpoint.sourceforge.net/
import fixedpoint

################################################################################
## FIXED POINT
################################################################################

def fixed_conv_in_imprecise(anything):
    # Imprecise fixed point I/O is implemented in typeconv_fixed_stdlib.py.
    raise NotImplementedError()

def fixed_conv_in_precise((val, scale)):
    # Allow implicit param conv:
    if val is None or isinstance(val, types.StringType):
        return val

    valCopy = val.copy()
    valCopy.set_precision(abs(scale))
    # We've taken care of the rounding via valCopy, so we can simply pass a
    # string representation of the rounded valCopy to the database engine.
    return str(valCopy)


def fixed_conv_out_imprecise(anything):
    # Imprecise fixed point I/O is implemented in typeconv_fixed_stdlib.py.
    raise NotImplementedError()

def fixed_conv_out_precise(x):
    if x is None:
        return None

    ############################################################################
    # Performance Notes:
    ############################################################################
    # This implementation is faster than these alternatives, including:
    # - Creating a FixedPoint with the scaled integer x[0] and then dividing
    #   that by a power of 10, even when the power of ten is cached, as in:
    #   def conv(x): # time: 153%
    #       absScale = abs(x[1])
    #       return fixedpoint.FixedPoint(x[0], absScale) / _tenTo[absScale]
    # - Dividing x[0] by the appropriate power of ten using divmod, then
    #   creating a string from the results and a FixedPoint from that string,
    #   as in:
    #   def conv(x): # time: 105%
    #       absScale = abs(x[1])
    #       return fixedpoint.FixedPoint('%d.%d' % divmod(x[0], _tenTo[absScale]), absScale)
    ############################################################################
    valAsRawText = str(x[0])
    scale = x[1]
    valAsText = '%s.%s' % (valAsRawText[:scale], valAsRawText[scale:])
    return fixedpoint.FixedPoint(valAsText, abs(scale))
